# -*- coding: utf-8 -*-

import subprocess
import re
import importlib.metadata


# 读取 requirements.txt 文件
def read_requirements(file_path):
    try:
        with open(file_path, 'rb') as f:
            content = f.read()
            try:
                content = content.decode('utf-8')
            except UnicodeDecodeError:
                import chardet
                encoding = chardet.detect(content)['encoding']
                content = content.decode(encoding)
        return content.splitlines()
    except FileNotFoundError:
        print(f"错误：未找到 {file_path} 文件。")
        return []


# 尝试安装指定版本的模块，如果失败则尝试安装最新版本
def install_module(module, version=None):
    try:
        if version:
            command = f"pip install {module}=={version}"
        else:
            command = f"pip install {module}"
        subprocess.run(command, shell=True, check=True)
        return True
    except subprocess.CalledProcessError:
        return False


# 主函数
def main():
    requirements_file = 'requirements.txt'
    requirements = read_requirements(requirements_file)
    replaced_versions = {}
    skipped_modules = []

    # 第一轮安装
    for requirement in requirements:
        match = re.match(r'(\w+)(==(\S+))?', requirement)
        if match:
            module = match.group(1)
            version = match.group(3)

            if version:
                if not install_module(module, version):
                    # 尝试安装最新版本
                    if install_module(module):
                        try:
                            latest_version = importlib.metadata.version(module)
                            replaced_versions[module] = (version, latest_version)
                        except importlib.metadata.PackageNotFoundError:
                            skipped_modules.append(requirement)
                    else:
                        skipped_modules.append(requirement)
            else:
                if not install_module(module):
                    skipped_modules.append(requirement)

    # 第二轮安装
    second_attempt_failed = []
    for requirement in skipped_modules:
        match = re.match(r'(\w+)(==(\S+))?', requirement)
        if match:
            module = match.group(1)
            version = match.group(3)

            if version:
                if not install_module(module, version):
                    if not install_module(module):
                        second_attempt_failed.append(requirement)
            else:
                if not install_module(module):
                    second_attempt_failed.append(requirement)

    # 总结
    print("版本替换情况：")
    for module, (old_version, new_version) in replaced_versions.items():
        print(f"{module}: {old_version} -> {new_version}")

    print("\n两轮安装后仍未找到并跳过安装的模块：")
    for module in second_attempt_failed:
        print(module)


if __name__ == "__main__":
    main()